/*
 *
 * @(#)BufferCanvas.java 0.1 97/08/11
 *
 * Copyright (c) 1997 Ginger Booth.  All Rights Reserved.
 *
 * @history: original
 * @author Ginger Booth
 * @version 0.1, August 1997
 *
 * @version 3.0, November2000 : Swing/Java 1.2/1.3 support, using
 * BufferCanvas as a place to localize new graphics support, since
 * most of my graphics classes build on BufferCanvas. At some point,
 * may wish to split the JFC utilities from the basic BufferCanvas
 * thing. And use the fact that components are double-buffered in JFC...
 */
package courseware.jfcdep;

/** A double-buffered canvas.  Draw offscreen.
*/
import java.awt.*;
import javax.swing.*;
import courseware.util.*;
import courseware.jfcdep.*;

public class BufferCanvas extends javax.swing.JComponent {
    protected Image offscreen;  // draw offscreen to cut the flicker
    protected Graphics offg;    // the graphics context of the offscreen image
                                // min=preferred size, requested
    protected Dimension prefSize = new Dimension();
                                // am is the current reality
    private Dimension amSize   = new Dimension();

    public final static float DEFAULT_CIRCLE_THICKNESS = 1.8f;
    public final static float DEFAULT_LINE_THICKNESS = 1.8f;

    public BufferCanvas(int prefsizex, int prefsizey) {
        prefSize      = new Dimension(prefsizex, prefsizey);
        amSize        = new Dimension(-1, -1);
    }

    public BufferCanvas(Dimension d) {
        prefSize      = new Dimension(d.width, d.height);
        amSize        = new Dimension(-1, -1);
    }
    public BufferCanvas() {this(1,1);}

    /** draw a -nice- circle in Swing--thickness 1.8 is very nice. Note
     *  that the floats are pixel coordinates. Using floats does keep the
     *  circles from jerking as much as they grow, because antialiased
     *  circles don't have to move a full pixel as the radius increases.
     */
    public void drawCircle(float ulx, float uly,
                           float diameter, float thickness, Color c)
    {
      Graphics2D g2 = (Graphics2D) getOffscreen();
      g2.setColor(c);
      g2.setStroke(new BasicStroke(thickness));
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      Shape circle = new java.awt.geom.Ellipse2D.Float(ulx, uly, diameter, diameter);
      g2.draw(circle);
    }
    public void fillCircle(float ulx, float uly,
                           float diameter, Color c)
    {
      Graphics2D g2 = (Graphics2D) getOffscreen();
      g2.setColor(c);
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      Shape circle = new java.awt.geom.Ellipse2D.Float(ulx, uly, diameter, diameter);
      g2.fill(circle);
    }
    public static void fillCircle(Graphics g, float ulx, float uly,
                           float diameter, Color c)
    {
      Graphics2D g2 = (Graphics2D) g;
      g2.setColor(c);
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      Shape circle = new java.awt.geom.Ellipse2D.Float(ulx, uly, diameter, diameter);
      g2.fill(circle);
    }

    /** antialiased text is not great looking on small fonts... */
    public void drawString(Color color, String s, int x, int y, boolean antialias) {
      Graphics2D g2 = (Graphics2D) getOffscreen();
      g2.setColor(color);
      if (antialias) {
        g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        g2.drawString(s, x, y);
      } else {
        g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
        g2.drawString(s, x, y);
      }
    }
    /** antialias off--if antialias is on for anything else, need to call this
     *  rather than g.drawString, to turn antialias off before drawing string.
     */
    public void drawString(Color color, String s, int x, int y) {
      Graphics2D g2 = (Graphics2D) getOffscreen();
      g2.setColor(color);
      g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
      g2.drawString(s, x, y);
    }
    /** lines look much better antialiased... */
    public void drawLine(Color c, int x0, int y0, int x1, int y1, float thickness) {
      Graphics2D g2 = (Graphics2D) getOffscreen();
      g2.setColor(c);
      g2.setStroke(
        new BasicStroke(thickness, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL)
      );

      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      g2.drawLine(x0, y0, x1, y1);
    }
    /** lines look much better antialiased...but unless want a globby/thin bit
     *  at each connected line segment, need a polyline...nah, too SLOW
     */
    public void drawLine(Color c, int[] x, int[] y, int npts, float thickness) {
      Graphics2D g2 = (Graphics2D) getOffscreen();
      g2.setColor(c);
      g2.setStroke(
        new BasicStroke(thickness)
      );

      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      g2.drawPolyline(x, y, npts);
    }
    // for DaisyBall's convenience
    static public void drawLine(Graphics g, Color c, int[] x, int[] y, int npts, float thickness) {
      Graphics2D g2 = (Graphics2D) g;
      g2.setColor(c);
      g2.setStroke(
        new BasicStroke(thickness)
      );

      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      g2.drawPolyline(x, y, npts);
    }
    // for doc purposes--this is even SLOWER than polyline in Java1.2
    /*
    static public void drawSegments(Graphics g, Color c, int[] x, int[] y, int npts, float thickness) {
      Graphics2D g2 = (Graphics2D) g;
      g2.setColor(c);
      g2.setStroke(
        new BasicStroke(thickness)
      );

      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      for (int i=0; i<npts-1; i++) {
        g2.drawLine(x[i], y[i], x[i+1], y[i+1]);
      }
    }
    */
    static public void fillPoly(Graphics g, Color c, int[] x, int[] y, int npts) {
      Graphics2D g2 = (Graphics2D) g;
      g2.setColor(c);
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      g2.fillPolygon(x, y, npts);
    }

    // for DaisyBall's convenience
    static public void drawLine(Graphics g, Color c,
      int x0, int y0, int x1, int y1, float thickness)
    {
      Graphics2D g2 = (Graphics2D) g;
      g2.setColor(c);
      g2.setStroke(
        new BasicStroke(thickness)
      );

      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      g2.drawLine(x0, y0, x1, y1);
    }


    // force all the graphics to start over
    public void reset() {
        if (offscreen != null) {
            offscreen.flush();
            offscreen = null;
        }
        amSize.width = -1;
    }

    /** Ensure we have an offscreen image to draw on, and that the scaling
     *  matches its dimensions.
     */
    synchronized public Graphics getOffscreen() {
        Dimension s = getSize();
        if (s.width < 1) s.width = 1;
        if (s.height < 1) s.height = 1;
        if ( (s.width  != amSize.width) ||
             (s.height != amSize.height)) {

            if (offscreen != null) {
                offscreen.flush();
            }

            offscreen     = createImage(s.width, s.height);

            if (offscreen != null) {
              amSize.width  = s.width;
              amSize.height = s.height;
              offg          = offscreen.getGraphics();
            }
        }
        return offg;
    }

    /** Ensure we have a NEW offscreen image to draw on.
     */
    protected Graphics newOffscreen() {
        Dimension s = getSize();

        if (offscreen != null)
            offscreen.flush();

        amSize.width  = s.width;
        amSize.height = s.height;
        offscreen     = createImage(amSize.width, amSize.height);
        offg          = offscreen.getGraphics();

        return offg;
    }

    // put offscreen graphics onscreen
    public void update(Graphics g) { paint(g); }

    // put offscreen graphics onscreen
    public void paint(Graphics g) {
        getOffscreen();
        if (offscreen != null)
           g.drawImage(offscreen, 0, 0, this);
    }
    // put offscreen graphics onscreen
    public void onscreen() {
        if (offscreen != null)
           getGraphics().drawImage(offscreen, 0, 0, this);
    }

    synchronized public void setSize(int w, int h)   {
        Dimension d = new Dimension(w, h);
        this.setSize(d);
    }

    synchronized public void setSize(Dimension d)   {
        prefSize = d;
        super.setSize(d);
        getOffscreen();
    }

   synchronized public void setBounds(int x, int y, int width, int height)   {
        Dimension d = new Dimension(width, height);
        prefSize = d;
        super.setBounds(x,y,width,height);
        getOffscreen();
    }

    public Dimension getPreferredSize() { return prefSize; }
    public void setPreferredSize(Dimension d) {prefSize = d; amSize=new Dimension(-1, -1);}
    public Dimension getMinimumSize()   { return prefSize; }
    public Dimension getCurrentSize()   { return amSize; }

    /** Release the graphics allocation. */
    public void finalize() {
        if (offscreen != null)
            offscreen.flush();
        offscreen = null;
        try {
            super.finalize();
        } catch (Throwable t) {};
    }

} /* BufferCanvas */
