package courseware.jfcdep;

import java.awt.*;
import javax.swing.*;
import java.net.URL;
import java.util.*;
import java.applet.*;
import java.awt.image.*;

/**
 * Title:        Courseware
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Yale Inst. Biospheric Studies
 * @author Ginger Booth
 * @version 2.0
 *
 * Someplace to put pesky resources like bg, bgImage, how to
 * draw a string in the right place, etc.
 */

public class CWGlobals {
  static public Color foreground;
  static public Color background;
  static public Applet applet; // may not be set....
  static public Font theFont;
  static public boolean haveComments;
  static public Paint bgPainter;
  static public final int PRESWING = 0;
  static public final int SWING    = 1;
  static public int version = SWING;

  static public Vector buttonImages = null; // if null, none avail.

  static public void drawBackground(Graphics g, Component c) {
    Rectangle bgRect = new Rectangle(0,0,c.getWidth(), c.getHeight());
    drawBackground(g, bgRect);
  }
  static public void drawBackground(Graphics g, Rectangle bgRect) {
    if (bgPainter == null) {
      g.setColor(background);
      g.drawRect(bgRect.x, bgRect.y, bgRect.width, bgRect.height);
      return;
    }
    Graphics2D g2 = (Graphics2D) g;
    g2.setPaint(bgPainter);
    g2.fill(bgRect);
  }


  static public void setButtonImages(int group,
    String plainFile, String pressedFile,
    Applet applet)
  {
    Image plainI = null;
    Image pressI = null;
    try {
      URL plainU = new URL(applet.getDocumentBase(), plainFile);
      URL pressU = new URL(applet.getDocumentBase(), pressedFile);
      plainI = applet.getImage(plainU);
      pressI = applet.getImage(pressU);
      MediaTracker waiter = new MediaTracker(applet);
      waiter.addImage(plainI, 0);
      waiter.addImage(pressI, 1);
      try {

        waiter.waitForAll();
        setButtonImages(group, plainI, pressI);
      } catch (InterruptedException e) {
        System.err.println("skin button load images interrupted");
      }
      if (waiter.isErrorAny()) {
        System.err.println("skin button load images error");
      }
    } catch (Exception e) {
      System.err.println("failed to load button skins: " + e);
    }
  }
  static public void setButtonImages(int group, Image plain, Image pressed) {
    if (buttonImages == null)
      buttonImages = new Vector(1,1);
    int hadGroup = findButtonGroupIndex(group);
    ButtonImageSet newSet;
    if (hadGroup >= 0) {
      newSet = (ButtonImageSet) buttonImages.elementAt(hadGroup);
      newSet.plain = plain;
      newSet.pressed = pressed;
    } else {
      newSet = new ButtonImageSet(group, plain, pressed);
      buttonImages.add(newSet);
    }
  }
  static public ButtonImageSet getButtonImages(int group) {
    if (buttonImages == null)
      return null;
    int groupIndex = findButtonGroupIndex(group);
    if (groupIndex < 0)
      return null;
    return (ButtonImageSet) buttonImages.elementAt(group);
  }

  static public int findButtonGroupIndex(int group) {
    if (buttonImages == null)
      return -1;
    for (int i=0; i<buttonImages.size(); i++) {
      ButtonImageSet bis = (ButtonImageSet) buttonImages.elementAt(i);
      if (bis.group == group)
        return i;
    }
    return -1;
  }
  /** Print an object as a unique id for debug purposes. */
  public static String pointerString(Object o) {
    return (o==null)? "null" : (new Integer(o.hashCode())).toString();
  }

  static public void setBackgroundImage(String imagefile, Applet applet) {
    Image bgImage;
    BufferedImage bgBufferImage;
    TexturePaint bgPaint;
    try {
      URL bgU = applet.getClass().getClassLoader().getResource(imagefile);
      bgImage = applet.getImage(bgU);
      MediaTracker waiter = new MediaTracker(applet);
      waiter.addImage(bgImage, 0);
      try {
        waiter.waitForAll();
      } catch (InterruptedException e) {
        System.err.println("skin bg load images interrupted");
      }
      if (waiter.isErrorAny()) {
        System.err.println("skin bg load images error");
        return;
      }

    } catch (Exception e) {
      System.err.println("failed to load background skin: " + e);
      return;
    }
    bgBufferImage = new BufferedImage(bgImage.getWidth(applet),
        bgImage.getHeight(applet),
        BufferedImage.TYPE_INT_RGB);
    Graphics2D g = bgBufferImage.createGraphics();
    g.drawImage(bgImage, 0, 0, applet);
    bgPaint = new TexturePaint(bgBufferImage,
      new Rectangle(0,0,bgImage.getWidth(applet), bgImage.getHeight(applet)));
    // don't dither--the browser doesn't, and inside, Java does even if
    // use Web palette for bg gif, so force it off....

    CWGlobals.bgPainter = bgPaint;
  }

  /** For the application aot applet versions... */
  static public void setBackgroundImage(String imagefile) {
    System.err.println("setBackgroundImage sans applet not yet supported.");
  }
  static public void setButtonImages(int group, String plain, String pressed) {
    System.err.println("setButtonImages sans applet not yet supported.");
  }

}