package courseware.jfcdep;

import javax.swing.plaf.metal.MetalButtonUI;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import javax.swing.plaf.basic.*;
import java.util.*;
import java.applet.Applet;
import courseware.util.*;
import courseware.jfcdep.*;

/**
 * Title:        Courseware
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Yale Inst. Biospheric Studies
 * @author Ginger Booth
 * @version 3.0
 */

public class SkinnedButtonUI extends MetalButtonUI {
  private final static SkinnedButtonUI skinnedButtonUI = new SkinnedButtonUI();

  static public int spacing = 1; // this does nothing somehow...looks best that way
  static public int insetText = 3;
  private final EmptyBorder border = new EmptyBorder(spacing, spacing, spacing, spacing);
  private static Rectangle viewRect = new Rectangle();
  static public Object myKey = "SkinButtonGroup"; // value = Integer

  public static ComponentUI createUI(JComponent c) {
    return skinnedButtonUI;
  }

  static private ButtonImageSet getImages(JComponent c) {
    ButtonImageSet imageSet = null;
    Object propValue;
    if ((propValue = c.getClientProperty(myKey)) != null) {
      int group = ((Integer) propValue).intValue();
      imageSet = CWGlobals.getButtonImages(group);
    }
    if (imageSet == null) {
      imageSet = CWGlobals.getButtonImages(0);
    }
    return imageSet;
  }

  /** mod from BasicButtonUI */
  public void installDefaults(AbstractButton b) {
    // from BasicButtonUI
    defaultTextIconGap = spacing;
    defaultTextShiftOffset = 0;
    b.setOpaque(true);
    b.setMargin(new Insets(spacing, spacing, spacing, spacing));

    b.setFocusPainted(false);
    b.setRolloverEnabled(true);
    b.setBorder(border);
  }

  public Dimension getPreferredSize(JComponent c) {
    Dimension d = super.getPreferredSize(c);
    d.width  += 2*insetText;
    return d;
  }

  /** Perhaps no-one paints button because it's background-color... */
  public void paint(Graphics g, JComponent c) {
    ButtonImageSet imageSet = getImages(c);
    if (imageSet == null) {
      super.paint(g, c);
      return;
    }

    CWGlobals.drawBackground(g, c);

    g.drawImage(imageSet.plain, 0,0, c.getWidth(),c.getHeight(), c);
    super.paint(g, c);
  }


  /** override from MetalButtonUI */
  protected void paintButtonPressed(Graphics g, AbstractButton b) {
    ButtonImageSet imageSet = getImages(b);
    if (imageSet == null) {
      super.paintButtonPressed(g, b);
      return;
    }
    g.drawImage(imageSet.pressed, 0, 0, b.getWidth(), b.getHeight(), b);
  }

  protected void paintText(Graphics g, JComponent c,
    Rectangle textRect, String text)
  {
    AbstractButton b = (AbstractButton) c;
    ButtonModel model = b.getModel();

    /*** paint the text normally whether button enabled or not */
    g.setColor(b.getForeground());
    g.setFont(CWGlobals.theFont);
    // textRect was set from not-my-choice of font
    // plus, I want them centered.
    FontMetrics fm = g.getFontMetrics();
    Rectangle2D rect2d = fm.getStringBounds(text,g);
    textRect.width = Math.round(Math.round(rect2d.getWidth()));
    textRect.height = Math.round(Math.round(rect2d.getHeight() ) );
    int xorig = (c.getWidth() - textRect.width) / 2;
    BasicGraphicsUtils.drawString(g,text, model.getMnemonic(),
        xorig, textRect.y + fm.getAscent());
  }
}
