//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999-2001, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator

/** vfb, 001227, added -restart N switch FOR BACSIM USE. Do NOT use for Gecko
 *  without further thought, as it only changes the starting run number from
 *  0 to N (-restart N) and sets a restartFlag, and tells UI to append to output
 *  files. No special handling for:
 *   a. the logic of resuming mid-Run
 *   b. resetting the random seed stream!!!
 */


package gecko;

import courseware.util.*;
import courseware.plot.*;
import courseware.jfcdep.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;
import java.lang.reflect.*;

import stats.*;

public class GeckoApplication implements GeckoDriver {
  static private Simulator simulator;
  static private ParamList protocol;
  static private ParamMgr protocolReader;
  static private BatchUI ui;
  static private String protocolFile;
  static private boolean batchFlag;
  static private boolean restartFlag;
  static private boolean bacsimFlag = false;
  static private String fileStem = "gecko";
  static String geckoVersion = "Gecko dev 2.1, Spring'00";
  static CWFrame frame;
  static Globals globals = new Globals(); // need to pass an instance....
  static private int mainWidth = 600;
  static private int mainHeight = 480;
  static private ExecButton scenarioButton;
  static private String scenarioLabel = "Scenarios";
  static private boolean firstTime = true;
  static private int numRuns = 1;
  static private int run;
  static private int thisRun; // copy because threads out of whack...
  static private int restartRun; // 0 based!!!

  public GeckoApplication(boolean batchFlag, boolean restartFlag, int restartTime, int restartRun, String protocolFile) {
    this.protocolFile = protocolFile;
    this.batchFlag    = batchFlag;
    this.restartFlag  = restartFlag;
    Globals.setGeckoDriver(this);
    if (restartFlag) {
      this.restartRun = restartRun;
      Globals.setCurrentTime(restartTime);
    } else {
      this.restartRun = 0;
      Globals.setCurrentTime(0);
    }
    init();
    firstTime = true;
  }
  public Container getContentPane(){return frame.getContentPane();}


  public boolean isRestart() {return restartFlag;}
  public void setBacsim(boolean b) {bacsimFlag = b;}
  public boolean isBacsim() {return bacsimFlag;}

  public void setBackgroundImage(String s) {
    CWGlobals.setBackgroundImage(s);
  }
  public void setBackgroundImageOff() {
    CWGlobals.bgPainter = null;
  }

  public void setButtonImages(String plain, String pressed) {
    CWGlobals.setButtonImages(0, plain, pressed);
  }

//  public Container getContentPane() {return frame.getContentPane();}

  /** JFC waldos */
  public Component addComponent(Component comp) {
    return frame.add(comp);
  }
  public void addComponent(String constraint, Component comp) {
    frame.add(constraint, comp);
  }

  public void comment(String msg) {
    ui.log(msg);
  }

  public void setNumRuns(int i) {numRuns = i;}
  public int getNumRun() {return thisRun;}

  /** This weirdness is because Netscape won't allow finding a method for the
   *  module and invoking it from CourseWareApplet. Hence, we need to execute
   *  a method named in module's interface to get at them.
   */
  public void moduleMethod(String methodName) {
    System.err.println("moduleMethod: no such method: " + methodName);
  }
  public void moduleMethodArgs(String methodName, String args) {
    System.err.println("moduleMethod: no such method: " + methodName + "("+args+")");
  }


  public void doLayout() {
    if (batchFlag)
      return;
    frame.doLayout();
  }
  public void setOutputFileStem(String s){
    fileStem = s;
    if (ui != null)
      ui.setFileStem(fileStem);
  }
  public void setWidth(int w){mainWidth = w;}
  public void setHeight(int h){mainHeight = h;}
  public void setOwner(CourseWare cw) {}
  public CourseWare getOwner() {return this;}
  public Container getContainer() {return frame;}

  public void init() {
    readProtocol();

    // recall UI is for statplots. Main frame is for the world & buttons.
    makeUI();
    simulator = new Simulator();
    Globals.setSimulator( simulator );
    Globals.setUI( ui );
    Stats statMinder = new Stats(ui);
    Globals.setStats(statMinder);

    ui.init();
    addExitButton(); // In File | Exit's normal posn in button lineup.

    statMinder.init();

    simulator.setDriver(frame);
    simulator.makeWorld();
    makeControlButtons();

    // Apply protocol to self first, then build the
    // substructure, then bind & apply the protocol to substructure.

    // GINGER:  I DO NOT LIKE THIS, NO I DON'T.....
    protocol = protocolReader.getParameters();
    protocol.bindAllObjects(this.getClass(), this);
    protocol.apply();

    bindParams(protocol);
    protocol.apply();

    postParams();

    simulator.postParams(); // does post-params init on world
  }

  public void finalize() {
    Globals.getUI().finalize();
  }

  private void postParams() {
    if (batchFlag)
      return;
    frame.setSize(mainWidth, mainHeight);
    Toolkit tk = frame.getToolkit();
    Dimension screensize = tk.getScreenSize();
    frame.setLocation(screensize.width - mainWidth - 1, 0);
    frame.repaint();
  }

  public void bindParams(ParamList p) {
    p.bindAllObjects(this.getClass(), this);
    p.bindAllObjects(CourseWare.class, this);
    if (!batchFlag)
      p.bindAllObjects(Globals.getWorldDisplay().getClass(), Globals.getWorldDisplay());
    p.bindAllObjects(simulator.getClass(), simulator);
  }

  private void makeUI() {
    if (batchFlag) {
      ui = new BatchUI(fileStem, protocolFile, globals);
    } else {
      frame = new CWFrame();
      initLayout(frame);
      frame.addWindowListener(new WindowAdapter(){
        public void windowClosing(WindowEvent e){
          System.exit(0);
        }
      });
      ui = new ApplicationGUI(fileStem, protocolFile, frame, globals);
      frame.setTitle(geckoVersion);
    }
    ui.setAppendMode(restartFlag);
    ui.log("// " + geckoVersion);
    ui.log("// Run date: " + new Date());
  }

  private void initLayout(Frame frame) {
    if (batchFlag) return;

    ModuleLayout ml = new ModuleLayout(1);
    int pad = CoursePanel.getPadding();
    ml.setOptions(10, pad, pad, false);
    frame.setLayout(ml);
  }

  private void readProtocol() {
    try {
      protocolReader = new ParamMgr(new URL("file:"+protocolFile), this);
    } catch (Exception e) {
      e.printStackTrace();
      System.err.println("*** Error reading protocol:" + protocolFile);
      System.exit(0);
    }
  }

  public ParamList getParams() {return protocol;}
  public Param getParam(String paramName) {return protocolReader.getParameter(paramName);}


  public void clear() {
    simulator.clear();
  }

  public Component add(Component c) {frame.add(c); return c;}
  public Component add(String constraint, Component c) {frame.add(constraint, c); return c;}
  public LayoutManager getLayout(){return frame.getLayout();}

  private void addExitButton() {
    if (batchFlag) return;
    ParamList pl = new ParamList("Exit");
    try {
      pl.setOwner(this);
      pl.setMethod("exit");
    } catch (NoSuchMethodException e) {}
    makeButtonCommon(pl);
  }

  // has to be public, or can't be executed via Method call...
  public void exit() {
    Globals.getUI().finalize();
    System.exit(0);
  }

  /** This one's the equivalent of CourseWareAdapter's protocol makeButton */
  public void makeButton(String paramListName) {
    if (batchFlag) return;
    Param theList = protocolReader.getParameter(paramListName);
    if (theList == null) {
        System.err.println("Couldn't find params to makeButton(" +
            paramListName + ")");
        return;
    }
    makeButtonCommon(theList);
  }

  /** used by makeButton */
  private void makeButtonCommon(Param theList) {
    // make the new button
    ExecButton eb = new ExecButton(theList.name, theList,
        ParamDialog.WITH_BOTH_BUTTONS);
    ModuleLayout ml = (ModuleLayout) frame.getLayout();

    // check if we already have such a button....
    int numComps = frame.getComponentCount();
    for (int i = 0; i<numComps; i++) {
        Component c = frame.getComponent(i);
        if (c instanceof ExecButton) {
            ExecButton ec = (ExecButton) c;
            if (theList.name.equals(ec.getLabel())) {
                frame.add(eb, ModuleLayout.BUTTONLIST_STRING );
                frame.remove(ec);
                eb.setVisible(true);
                return;
            }
        }
    }
    // if still here, we're adding rather than replacing button.
    frame.add(eb, ModuleLayout.BUTTONLIST_STRING);
    eb.setVisible(true);
  }

  public void makeControlButtons() {
    if (batchFlag) return;

    scenarioButton = protocolReader.makeScenarioButton(scenarioLabel);
    frame.add(scenarioButton, ModuleLayout.BUTTONLIST_STRING);

    GeckoDriverCommon.makeControlButtons(frame, simulator);
  }

  /** Set background of quaddisplay and plotters.*/
  public void setBackground(Color c) {
    if (batchFlag) return;
    GeckoDriverCommon.setBackground(frame, c);
  }
  /** Set foreground of quaddisplay and plotters.*/
  public void setForeground(Color c) {
    // if uncomment this, some but not all buttons get diff color text....
    // super.setForeground(c);
    if (batchFlag) return;
    Globals.getWorldDisplay().setForeground(c);
    Plotter.setDefForegroundColor(c);
  }
  public void setButtonBackground(Color c) {
    MatchedButton.background = c;
  }
  public void setButtonForeground(Color c) {
    MatchedButton.foreground = c;
  }




  public void setGridColor(Color c) {
    if (batchFlag) return;
    Plotter.setDefGridColor(c);
  }

  public void update (Graphics g) { frame.paint(g); }
//  public void update (Graphics g) { frame.paintComponents(g); }

  public void setRandomSeed(int s){Globals.setRandomSeed(s);}

  public void start() {
    // reassert random seed now so that multi-runs can be restarted in the middle.
    // random numbers have been consumed already, but they have no effect, so it's
    // safe to reset the generator at this point.
    Globals.setRandomSeed(Globals.getRandomSeed());
    // now start 1-n runs
    for (run=restartRun; run<numRuns; run++) {
      ui.log("// Run number " + run + " begins: " + new Date());
      ui.log("// setRandomSeed(" + Globals.getRandomSeed() + ")");
      if (firstTime) {
        firstTime = false;
        if (!batchFlag) {
          frame.doLayout();
          Globals.getWorldDisplay().centerPicture(Globals.getWorldMcr());
        }
        synchronized (simulator) {
          // removed, vfb 010111, was calling twice in practice...
          //      simulator.postParams();
          // and that this doesn't work in postparams is probably why....
          thisRun = run;
          simulator.start();
        }
      } else {
        bindParams(protocol = protocolReader.getParameters());
        clear();
        Globals.setCurrentTime(0);
        protocol.apply();
        if (!batchFlag)
          frame.doLayout();
        simulator.postParams();
        thisRun = run;
        simulator.start();
      }
      // vfb, 010114, this used to be only if batch flag, but run# marched on
      if (batchFlag) {
        try {
          simulator.getThread().join();
        } catch (InterruptedException e) {}
        finally {
          ui.log("// Run ends: " + new Date());
          setRandomSeed(Globals.getRandomSource().nextInt());
        }
      }
    }

    if (batchFlag) {
      ui.finalize();
      System.exit(0);
    }
  }

  public void stop() {
    simulator.pause();
  }

  public void addAllSpeciesTo(String paramListName) {
    GeckoDriverCommon.addAllSpeciesTo(this, paramListName);
  }
  public void addStatsTo(String paramListName, boolean includeSites) {
    GeckoDriverCommon.addStatsTo(this, paramListName, includeSites);
  }
  public void addStatsTo(String paramListName) {
    GeckoDriverCommon.addStatsTo(this, paramListName, true);
  }
}
