//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator

/** Since GeckoDriver has to be an interface, here's a place to put routines
 *  implementing the common stuff for GeckoCourseWare and GeckoApplication.
 */

package gecko;
import courseware.util.*;
import courseware.plot.*;
import courseware.jfcdep.*;
import java.awt.Color;
import java.awt.Container;
import java.util.Vector;
import stats.Plotter;

public class GeckoDriverCommon {

  private static void abutton(Container gd, Simulator sim, String label, String methodname) {
    try {
      ExecButton b = new ExecButton(label, sim, methodname);
      gd.add(b, ModuleLayout.BUTTONLIST_STRING);
      ((ModuleLayout) gd.getLayout()).setConstraint(b, 0, ModuleLayout.BUTTONLIST);
    } catch (Exception e) {}
  }

  public static void makeControlButtons(Container gd, Simulator sim) {
    abutton(gd, sim, "Pause", "pause");
    abutton(gd, sim, "Continue", "simulateContinue");
    abutton(gd, sim, "Restart", "restart");
    abutton(gd, sim, "Step 1", "simulateMore1");
    abutton(gd, sim, "Step 10", "simulateMore10");
    // debug
    // abutton(gd, sim, "DumpQuads", "dumpTree");
  }

  /** Set background of quaddisplay and plotters.*/
  public static void setBackground(Container gd, Color c) {
    gd.setBackground(c);
    Globals.getWorldDisplay().setBackground(c);
    Plotter.setDefBackgroundColor(c);
    CWGlobals.background = c;
  }

  /** Set foreground of quaddisplay and plotters.*/
  public static void setForeground(Container gd, Color c) {
    // if uncomment this, some but not all buttons get diff color text....
    // gd.setForeground(c);
    Globals.getWorldDisplay().setForeground(c);
    Plotter.setDefForegroundColor(c);
    CWGlobals.foreground = c;
  }

  public static void setGridColor(Color c) {
    Plotter.setDefGridColor(c);
  }

  public void addStatsTo(GeckoDriver gd, String paramListName) {
    addStatsTo(gd, paramListName, true);
  }

  public static void addStatsTo(GeckoDriver gd, String paramListName, boolean includeSites) {
    ParamList theList = (ParamList) gd.getOwner().getParam(paramListName);
    if (theList == null) {
      System.err.println("Couldn't find params " + paramListName );
      return;
    }
    // all those gizmos which have statistics
    Vector allSpecies = Species.getAllSpecies();
    Vector allComposites = Species.getAllCompositeSpecies();
    Vector allSites = Site.getAllSites();

    ParamList whichButton;
    for (int i=0; i<allComposites.size(); i++) {
      theList.add(whichButton = new ParamList());
      CompositeSpecies oneSpecies = ((CompositeSpecies)allComposites.elementAt(i));
      ParamList oneSpeciesList = oneSpecies.getStatParamList();
      oneSpeciesList.setName(oneSpecies.getName());
      whichButton.add(oneSpeciesList);
    }
    for (int i=0; i<allSpecies.size(); i++) {
      theList.add(whichButton = new ParamList());
      Species oneSpecies = ((Species)allSpecies.elementAt(i));
      ParamList oneSpeciesList = oneSpecies.getStatParamList();
      oneSpeciesList.setName(oneSpecies.getName());
      whichButton.add(oneSpeciesList);
    }
    if (includeSites) {
      for (int i=0; i<allSites.size(); i++) {
        theList.add(whichButton = new ParamList());
        Site oneSite = ((Site)allSites.elementAt(i));
        ParamList oneSpeciesList = oneSite.getStatParamList();
        whichButton.add(oneSpeciesList);
      }
    }
  }

  public static void addAllSpeciesTo(GeckoDriver gd, String paramListName) {
    ParamList theList = (ParamList) gd.getOwner().getParam(paramListName);
    if (theList == null) {
      System.err.println("Couldn't find params " + paramListName );
      return;
    }

    // want species at 3rd level down nested lists, for buttons bringing
    // up one menu per species...

    Vector allSpecies = Species.getAllSpecies();

    // members of listOfSpecies are 4th level nesting
    for (int i=0; i<allSpecies.size(); i++) {
      ParamList speciesButton = new ParamList();
      theList.add(speciesButton);
      Species oneSpecies = ((Species)allSpecies.elementAt(i));
      ParamList oneSpeciesList = oneSpecies.getParamList();
      oneSpeciesList.setName(oneSpecies.getName());
      speciesButton.add(oneSpeciesList);
    }
  }
}
