//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * 990329, vfb, original Java
 * Assorted utilities. Analogous to "Base" in SwarmGecko.
 */


package gecko;
import java.util.Random;
import stats.*;
import gecko.zoo.*;

public class Globals extends Timer {

  // static int now is in Timer
  // these objects are available for everyone to talk to. They are not
  // constants, however--ask for them any time you want to talk to them.
  // Especially non-obvious--you're talking to a different instance of
  // Simulator at non-obvious intervals.
  static private Quad theWorld;
  static private LandRoot landRoot; // hangs on world tree, but often referred to
  static private Quad plantRoot; // hangs on world tree, but often referred to
  static private Quad mobileRoot; // hangs on world tree, but often referred to
  static private QuadDisplay worldDisplay;
  static private Simulator simulator;
  static private int seed = (int) System.currentTimeMillis();
  static private Random randomSource = new Random((long) seed);
  static private Stats stats;
  static private UI ui;
  static private int statCycle; // stats are written every statCycle'th timesteps
  static private int seasonCycle = 0; // >0 if any
  static private int sortCycle = 10; // how often to requad the tree
  static private Resourceable resources = new GrassResources();
  static private GeckoDriver geckoDriver;
  static public String POP_PLOT_NAME           = "Population";
  static public String BIO_PLOT_NAME           = "Scaled Biomass";
  static public String BIO_TRANSFERS_PLOT_NAME = "Biomass Transfers";
  static public String BIO_RATE_PLOT_NAME      = "Biomass Rates";
  static public String BIO_THEFT_PLOT_NAME     = "Theft vs. Biomass";
  static public String BIO_GROWTH_PLOT_NAME    = "Growth vs. Biomass";
  static public String STAGE_FERT_PLOT_NAME    = "Per Capita Fertility";
  static public String STAGE_MORT_PLOT_NAME    = "Per Capita Mortality";
  static public String STAGE_PLOT_NAME         = "Population By Stage";

  // just a dodge to turn on debugging in class A when hit condition in class B
  // this should never be used unless debugging something that happens, say, in
  // timestep 93 in Quad.pickWin.
  static private boolean hitDebugCondition = false;

// GINGER:  remember, decided LandRoot should not be in charge of requadding.

  static public void setGeckoDriver(GeckoDriver appOrApplet) { geckoDriver = appOrApplet;}
  static public GeckoDriver getGeckoDriver(){return geckoDriver;}
  static public void setStats(Stats s) { stats = s; }
  static public Stats getStats() { return stats;}
  static public int getStatCycle() { return statCycle;}
  static public void setStatCycle(int timesteps) { statCycle = timesteps;}
  static public int getSortCycle() { return sortCycle;}
  static public void setSortCycle(int timesteps) { sortCycle = timesteps;}
  static public int getSeasonCycle() { return seasonCycle;}
  static public void setSeasonCycle(int timesteps) { seasonCycle = timesteps;}
  static public boolean isSeasonal() { return seasonCycle > 0;}
  static public boolean isSeasonNow() {
    if  (seasonCycle <= 0)
      return false;
    if (getCurrentTime() == 0)
      return false;
    // if season is 1900 long, starting at 0, then season -ends- at 1899
    // and season -begins- at 1900
    return ( (getCurrentTime() % seasonCycle) == (seasonCycle-1) );
  }
  static public void setUI(UI gui) { ui = gui; }
  static public UI getUI() { return ui;}
  static public void log(String s) {ui.log(s);}
  static public int getRandomSeed() {return seed;}
  /** Note: Yes, the random number generator uses longs. However. The ParamMgr
   *  cannot read longs correctly due to a fundamental flaw in StreamTokenizer
   *  and Double/Long. Note that though Double has a routine named "doubleToLongBits",
   *  it does not return the double interpreted as a long, but...something else.
   *  Rather than fix all that, which I'm not sure how to do, I seed with int.
   */
  static public void setRandomSeed(int rs) {seed = rs; randomSource.setSeed((long) seed);}
  static public Random getRandomSource(){return randomSource;}
  static public void setRandomSource(Random random) {randomSource = random;}
  static public double getRandomDouble(){return randomSource.nextDouble();}
  static public void setResources(Resourceable resourceLookup) {
    resources = resourceLookup;
  }
  static public Resourceable getResources() {
    return resources;
  }
  static public long getCurrentTime() { return now;}
  static public void setCurrentTime(int i) { now = i;}

  static public void setMaxPerQuad(int n){Quad.setMaxPerQuad(n);}
  static public Quad getWorldRoot() { return theWorld; }
  static public void setWorldRoot(Quad wRoot) { theWorld = wRoot; }
  static public Quad getPlantRoot() { return plantRoot; }
  static public void setPlantRoot(Quad pRoot) { plantRoot = pRoot; }
  static public LandRoot getLandRoot() { return landRoot; }
  static public void setLandRoot(LandRoot lRoot) { landRoot = lRoot; }
  static public Quad getMobileRoot() { return mobileRoot; }
  static public void setMobileRoot(Quad mRoot) { mobileRoot = mRoot; }
  static public QuadDisplay getWorldDisplay() { return worldDisplay; }
  static public void setWorldDisplay(QuadDisplay circleDrawer) { worldDisplay=circleDrawer; }
  static public void setSimulator(Simulator s) {simulator = s;}
  static public Simulator getSimulator() {return simulator;}
  static public Mcr getWorldMcr() {
    Mcr mcr = new Mcr();
    if ( (landRoot != null) && (!landRoot.isNullMcr()) ){
      mcr.setFrom(landRoot);
      return mcr;
    }
    if ( (theWorld != null) && (!theWorld.isNullMcr()) ){
      mcr.setFrom(theWorld);
      return mcr;
    }
    return null;

  }

  static public void setWorldMcr(double x0, double y0, double x1, double y1) {
    if (landRoot != null)
      landRoot.setMcr(x0, y0, x1, y1);
    else if (theWorld != null)
      theWorld.setMcr(x0, y0, x1, y1);
    else
      System.err.println("WARNING: no world yet to set mcr on");
  }

  static public void addLandQuad(Quad quad) { landRoot.addChild(quad); }

  static public LandRoot getTaxCollector() {return landRoot; }
  static public void setTaxCollector(LandRoot lr) {landRoot = lr;}

  /** 2d distance */
  static public double distance(double x1, double y1, double x2, double y2) {
    double a = x2 - x1;
    double b = y2 - y1;
    return(Math.sqrt(a*a + b*b));
  }
  /** 3d distance */
  static public double distance(double x1, double y1, double z1,
                                double x2, double y2, double z2) {
    double a = x2 - x1;
    double b = y2 - y1;
    double c = z2 - z1;
    return(Math.sqrt(a*a + b*b + c*c));
  }

  static public boolean samesign(double a, double b) {
    return (a * b >= 0);
  }

  /**
  // Returns [fromx, fromy] if from/to points are coincident or dist = 0.
  // But if it works, the purpose is to project a point at distance dist from
  // point fromx, fromy, in the direction of point tox, toy.  We don't care
  // whether we overshoot, that's just the direction to go in.  (Or opposite,
  // in the case of a negative dist.)
  */
  public static double[] ptToward(double fromx, double fromy,
                                  double tox, double toy, double dist)
  {
    double[] ans = new double[] { fromx, fromy };
    double wnorm = distance(fromx, fromy, tox, toy);
    if (wnorm == 0.0)
      return ans;
    dist = dist / wnorm;
    ans[0] += (tox - fromx) * dist;
    ans[1] += (toy - fromy) * dist;
    return ans;
  }
  /* 3d version. If couldn't do it, answer is the original fromx,y,z */
  public static double[] ptToward(double fromx, double fromy, double fromz,
                                  double tox, double toy, double toz, double dist)
  {
    double[] ans = new double[] { fromx, fromy, fromz };
    double wnorm = distance(fromx, fromy, fromz, tox, toy, toz);
    if (wnorm == 0.0)
      return ans;
    dist = dist / wnorm;
    ans[0] += (tox - fromx) * dist;
    ans[1] += (toy - fromy) * dist;
    ans[2] += (toz - fromz) * dist;
    return ans;
  }

  /** angle of vector pointing this way, random default if coincident */
  public static double angleToward(double fromx, double fromy,
                                   double tox, double toy)
  {
    double wnorm;
    double angle = Math.PI * 2.0 * randomSource.nextDouble();//random default
    double dx = tox - fromx;
    double dy = toy - fromy;

    wnorm = Math.sqrt(dx * dx + dy * dy); // hypotenuse...
    if (wnorm > 0.0) {
      wnorm = Math.abs(dx / wnorm);
      if (wnorm > 1.0)
        wnorm = 1.0;
      angle = Math.acos(wnorm);// first quadrant angle...
      if (dx < 0.0)
        angle = Math.PI - angle;
      if (dy < 0.0)
        angle = (2.0 * Math.PI) - angle;
    }
    return angle;
  }

  /** The volume of a critter is a (maybe) sphere,
    * and the volume of the sphere is (4/3)* pi * r**3.
    * Radius = (r according to volume) * scaler.
    * Scalev = %spherical volume (nonsphere).
   */
  public static double radius2volume(double r, double scaler, double scalev){
    // unscale r
    r = r / scaler;
    return scalev * (4.0 / 3.0) * Math.PI * r * r * r;
  }

  public static double radius2area(double r){return Math.PI * r * r;}

  public static double volume2radius(double v, double scaler, double scalev) {
    return scaler * Math.pow( (v * 0.75 / (Math.PI * scalev)), 0.333333 );
  }
  /** unscaled version */
  public static double radius2volume(double r) {
    // The volume of the sphere is (4/3)* pi * r**3.
    return (4.0 / 3.0) * Math.PI * r * r * r;
  }

  /** unscaled version */
  public static double volume2radius(double v) {
    // The volume of the sphere is (4/3)* pi * r**3.
    return Math.pow( (v * 0.75 / Math.PI), 0.333333333333333333 );
  }

  /** Print an object as a unique id for debug purposes. */
  public static String pointerString(Object o) {
    return (o==null)? "null" : (new Integer(o.hashCode())).toString();
  }

  /** Print an object as class+unique id for debug purposes. */
  public static String agentID(Object o) {
    String classname = "" + o.getClass();
    int period = classname.lastIndexOf('.');
    classname = classname.substring(period+1, classname.length());
    return (o==null)? "null" : classname + ":" + (new Integer(o.hashCode())).toString();
  }
}


