//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator

/** This class basically declares the resources used in Gecko's grassworld
 *  series of models. A resource is actually an int. This provides int-String
 *  conversion and a list and count. In SwarmGecko, the resources were simply
 *  constants in Globals.
 */


package gecko;

public class GrassResources implements Resourceable {
  static private int numResources = 5; // 4 + null resource?
  final static public int RESOURCE_NULL     = 0; // nullResource is an active "other".
  final static public int RESOURCE_SUNWATER = 1;
  final static public int RESOURCE_CARBOS   = 2;
  final static public int RESOURCE_PROTEINS = 3;
  final static public int RESOURCE_CPROTEINS= 4;
  final static String[] names = { "Null", "SunWater", "Carbos", "Proteins", "CProteins" };
  final static double[] bioScale = { 1.0,
                                     1.0,
                                     1.0,
                                     1.0, // goppermass=goppermass * spiderconversion...
                                     1.0   // spidermass=normalizer unit
                                    };

  public GrassResources() { }
  public void setBioScale(int resourceId, double scalefactor) {
    bioScale[resourceId] = scalefactor;
  }
  public double getBioScale(int resourceId) { return bioScale[resourceId];}
  public int getNumResources() {return numResources;}
  public int getNullResource() {return RESOURCE_NULL;}
  public void setNumResources(int num) {numResources = num;}
  /** A return value of -1 is invalid--not found. Case sensitive. */
  public int getResourceId(String name) {
    for (int i=0; i<numResources; i++) {
      if (names[i].equals(name))
        return i;
    }
    return -1;
  }
  /** Returns "Null" if id is out of bounds, AND for id 0. Unnecessary coincidence. */
  public String getResourceName(int id) {
    if ((id < 0) || (id >= numResources))
      return names[0];
    else return names[id];
  }
  public void setResourceName(int id, String name) {
    names[id] = name;
  }


  public String toString(int i) {return getResourceName(i);}
}