//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator

/** Unhook the sunwater/carbos/etc scheme to allow others to plug in easier.
 *  I -want- resource to be a simple integer index into Metabolism, rather than
 *  a true class, for performance reasons. Hence, providing a Resourceable is
 *  a way to attach other information to the true Resource (an int) without
 *  hurting performance. Actual instances of Resourceable beyond Gecko's grass
 *  worlds may have significantly more data attached to a resource besides a
 *  number/name/bioscale/count lookup. That's fine.
 *
 *  In Gecko 0.whatever, bioScale was a denominator (biomass/bioscale) attached
 *  to a Species. The justification for it being a Species attribute was that
 *  I set it by conversion efficiencies between resources, which certainly are
 *  a Species attribute. The divisor rather than multiplicative factor bit was
 *  simply confusing. At any rate, the gizmo's purpose is to scale biomasses of
 *  apples and oranges so you can put them on the same plot and
 *  be able to see anything. It's not actually true that biomasses scale as
 *  conversion efficiencies, anyway, since metabolism is just as much a cost of
 *  the conversion.
 *
 *  In other words, this implementation of bioScale means something different
 *  than old Gecko bioscales. Especially important, that it is a factor, not
 *  a divisor, so needs to be set to 1/oldval.
 *
 *  Note: Metabolism assumes that Resource id 0 is "other".
 *
 *  Note:  Resourceable is NOT a resource/food source. That is a ResourceQuad.
 *  I.e., this class does not -produce- resources, it only declares them.
 */

package gecko;

public interface Resourceable {
  public int getNumResources();
  public void setNumResources(int num);
  public int getResourceId(String name);
  public String getResourceName(int id);
  public void setBioScale(int resourceId, double scalefactor);
  public double getBioScale(int resourceId);
  public int getNullResource();
  public String toString(int i);
}
