//Title:        BacSim/Gecko
//Version:
//Copyright:    Copyright (c) 1997-2000, All rights reserved
//Author:       Jan Kreft, Kreft@cardiff.ac.uk
//Company:      University of Cardiff
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Bacterial Ecology Simulator (BacSim)
/**
 * 001228, vfb, someplace to stuff things that were in GeckoControlSwarm,
 * but not of Gecko...  BacGlobals.init() is the perfect place to
 * put BacSim run initialization stuff.
 */


package gecko.bacsim;
import java.io.*;
import java.util.*;
import gecko.*;
import gecko.zoo.*;
import stats.*;
import courseware.util.*;

public class BacGlobals {
  // from Matlab eps
  public final static double EPSILON = 2.2204e-16;

  static PrintWriter bacFile = null;
  static PrintWriter porFile = null;
  static PrintWriter diffFile = null;
  static boolean restartFlag = false;
  static boolean filesOpenedFlag = false;
  static int restartRun = 0;
  static int startTime = 0;
  static int startRun = 0;

  static int checkpointCycle = 100; // save once every 100 steps, unless overridden

  // I do not understand what the iniGeckoRuns and geckoRuns stuff is about yet....
  // seems easy enough to do without, but...maybe not...

  /** Can't access from protocol without
   *  a constructor (which is *only* called by the protocol!)
   */
  public BacGlobals() {
    Globals.setResources(new BacsimResources());
  }

  /** This routine is basically BacSim.init, but since it's come very late in
   *  BacSim's evolution, there are many things that would make more sense
   *  here but have not been migrated to here. It is called from
   *  Simulator.postParams, after Sites.hangAllSites, before centering the
   *  picture (based on landroot's mcr). In other words, the protocol's been
   *  read in, all params applied to all grandmas, a Diffuse has been
   *  instantiated into the world, time is 0. Could be any run of multiple
   *  or from ui inside Application or Applet.
   */
  static public void init() {
    // filesOpenedFlag is false only on the very first time BacGlobals.init
    // is called by Simulator. Thus, the first of the megarun, which could be
    // a restart...
    if (!filesOpenedFlag) {
      // IN HERE, check if this is a restart run, which will be true exactly
      // for ONE RUN at most. Note this is different between BacSim's
      // restartFlag and Gecko's. Gecko's will remain true for rest of job,
      // but BacSim's will not.
      restartFlag = Globals.getGeckoDriver().isRestart();
      if (restartFlag) {
        startRun = Globals.getGeckoDriver().getNumRun();
        startTime =(int)Globals.getCurrentTime();
      }

      openFiles();

      // put any other pre-init, all-init bookkeeping stuff here!

      filesOpenedFlag = true;

    } else {
      // nth time through, time's up, forget this was ever a restarted job.
      if ( (restartFlag) && (Globals.getGeckoDriver().getNumRun() == startRun) )
        // we've been called twice, so stay true, and read 'em in again...
        restartFlag = true;
      else
        restartFlag = false;
    }

    if (restartFlag) {
      // Gecko already told UI to append to logs in this case.
      // for reasons I don't understand, Diffuse sets rng at -end- of stepResources,
      // so best to leave it there. So all that remains is reading the input file,
      // I guess.
      checkInputFile(false);
    } else {
      // Jan's "bac_xxx_tttttt.in" scheme for rerunning same critters, new rng
      // after critters located on tree....
      checkInputFile(true);
    }
  }

  static public void setCheckpointCycle(int i) {checkpointCycle = i;}
  static public int getCheckpointCycle() {return checkpointCycle;}

  static public void saveRngState() {
    ObjectOutputStream file;

    String filename = ((BatchUI) Globals.getUI()).getFileStem();
    filename += new PrintfFormat("_%03ld_").sprintf(Globals.getGeckoDriver().getNumRun());
    filename += new PrintfFormat("%06ld.rng").sprintf(Globals.getCurrentTime());

    try {
      file = new ObjectOutputStream(new FileOutputStream(filename, false));
      file.writeObject(Globals.getRandomSource());
      file.close();
    } catch (Exception e) {
      Globals.log("WARNING: error saving random state, e=" + e);
    }
  }

  static public void readRngState() {
    ObjectInputStream file;

    String filename = ((BatchUI) Globals.getUI()).getFileStem();
    filename += new PrintfFormat("_%03ld_").sprintf(Globals.getGeckoDriver().getNumRun());
    filename += new PrintfFormat("%06ld.rng").sprintf(Globals.getCurrentTime());

    try {
      file = new ObjectInputStream(new FileInputStream(filename));
      Globals.setRandomSource((Random)file.readObject());
      file.close();
    } catch (Exception e) {
      Globals.log("ERROR: error reading random state from file " + filename);
      Globals.log("Exception was" + e);
      Globals.getSimulator().crashStop();
    }
  }


  /** came from geckoModelSwarm-hangCritters. If useIn is false, means look
   *  for a ".ml" file and fail if none found, else use a ".in" file if it
   *  exists, but exit peaceably if it doesn't. Continuation of a scheme
   *  Jan did in objc, since I can't think of any easy way to do better.
   */
  static private void checkInputFile(boolean useIn) {
    BufferedReader file;
    if (useIn) {
      file = getInputFile();
    } else {
      file = getMatlabFile();
    }
    if (file == null) {
      if (!useIn) {
        Globals.log("ERROR: Could not open agent-list file for reading, cannot restart.");
        Globals.getSimulator().crashStop();
      }
      return;
    }
    Globals.log("Warning: now reading in agent-list from file " + filename);
    // One pass through squashQuad inverts the agent list, and writing/reading
    // inverts too
    Bacillus.makeAgentsFromFile(file);
    Quad root = Globals.getMobileRoot();
    root.squash();
    // root.subQuad();  // was commented out in objc...because changes order...
    root.growMcr();
  }

  static public Bacillus findProgenitor(int process) {
    Vector slist = Species.getAllSpecies();
    Bacillus maybe;
    Species species;
    for (int i=0; i < slist.size(); i++) {
      species = (Species) slist.elementAt(i);
      Speciesable s = species.getProgenitor();
      if (s instanceof Bacillus) {
        maybe = (Bacillus) s;
        if (maybe.getProcess() == process)
          return maybe;
      }
    }
    return null;
  }

  static public boolean isRestartFlag() {return restartFlag;}
  static public void setRestartFlag(boolean b) {restartFlag = b;}


  static public PrintWriter getBacFile() {return bacFile;}
  static public PrintWriter getPorFile() {return porFile;}
  static public PrintWriter getDiffFile() {return diffFile;}
  static private String filename;

  static public BufferedReader getInputFile() {
    filename = ((BatchUI) Globals.getUI()).getFileStem();
    filename += new PrintfFormat("_%03ld_").sprintf(Globals.getGeckoDriver().getNumRun());
    filename += new PrintfFormat("%06ld.in").sprintf(Globals.getCurrentTime());
    try {
      BufferedReader file = new BufferedReader(new FileReader(filename));
      return file;
    } catch (Exception e) {
      return null;
    }
  }
  static public BufferedReader getMatlabFile() {
    filename = ((BatchUI) Globals.getUI()).getFileStem();
    filename += new PrintfFormat("_%03ld_").sprintf(Globals.getGeckoDriver().getNumRun());
    filename += new PrintfFormat("%06ld.ml").sprintf(Globals.getCurrentTime());
    try {
      BufferedReader file = new BufferedReader(new FileReader(filename));
      return file;
    } catch (Exception e) {
      Globals.log("Error opening file: " + filename + ", msg=" + e);
      return null;
    }
  }

  static private void openFiles() {
    String filename = ((BatchUI) Globals.getUI()).getFileStem();
    bacFile = open(filename + ".bac");
    porFile = open(filename + ".por");
    diffFile = open(filename + ".diff");
  }

  static public void flushFiles() {
    if (bacFile != null) {
      bacFile.flush();
      porFile.flush();
      diffFile.flush();
    }
  }
  
  static private PrintWriter open(String filename) {
    if (restartFlag) {
      // append
      try {
        return new PrintWriter(new FileWriter(filename, true), true);
      } catch (Exception e) {
        Globals.log("Error : " + e + " Opening output file : " + filename);
      }
    } else {
      // overwrite
      try {
        return new PrintWriter(new FileWriter(filename, false), true);
      } catch (Exception e) {
        Globals.log("Error : " + e + " Opening output file : " + filename);
      }
    }
    return null;
  }

}
