//Title:        BacSim/Gecko
//Version:
//Copyright:    Copyright (c) 1997-1999, All rights reserved
//Author:       Jan Kreft, Kreft@cardiff.ac.uk
//Company:      University of Cardiff
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Bacterial Ecology Simulator (BacSim)
/**
 * A place to declare resources and their scaling factors.
 * 991013, vfb, translated from the Swarm Objective-C BacSim source.
 */
package gecko.bacsim;
import gecko.*;

public class BacsimResources implements Resourceable {
  static private int numResources = 5; // 4 + null resource?
  public static final int OXYGEN  = 0;
  public static final int AMMONIA = 1;
  public static final int NITRITE = 2;
  public static final int NITRATE = 3;
  public static final int NULL_RESOURCE = 4; // Grassworlds have 0 as nullResource

  final static String[] names = { "Oxygen", "Ammonia", "Nitrite", "Nitrate", "Unknown" };
  final static double[] bioScale = { 1.0, 1.0, 1.0, 1.0 };

  public BacsimResources() { }
  public void setBioScale(int resourceId, double scalefactor) {
    bioScale[resourceId] = scalefactor;
  }
  public double getBioScale(int resourceId) { return bioScale[resourceId];}
  public int getNumResources() {return numResources;}
  public int getNullResource() {return NULL_RESOURCE;}
  public void setNumResources(int num) {numResources = num;}
  /** A return value of -1 is invalid--not found. Case sensitive. */
  public int getResourceId(String name) {
    for (int i=0; i<numResources; i++) {
      if (names[i].equals(name))
        return i;
    }
    return -1;
  }
  /** Returns "Null" if id is out of bounds, AND for id 0. Unnecessary coincidence. */
  public String getResourceName(int id) {
    if ((id < 0) || (id >= numResources))
      return names[NULL_RESOURCE];
    else return names[id];
  }

  public String toString(int i) {return getResourceName(i);}
}