//Title:        Gecko Woods
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  "mini-Sortie"
/**
 * 991130, vfb, original Java
 * A Site that generates a fragmented landscape collection of quality sites.
 * The regular case (n x m sites with spacing s) is easy. The random case...
 * argh. Let's debug the regular case....
 *
 * 991221, vfb, hook up the random fragment mode. One FragSite does one or the
 * other mode, not both.
 */
package gecko.woods;

import gecko.*;
import gecko.zoo.RandomMcrs;
import java.awt.Color;
import java.util.*;

public class FragSite extends Site {

  // parameters of a regular fragmented array
  int numx;
  int numy;
  int numFrags;
  double maxq;
  double minq;
  boolean gradient; // if true, max->min across all frags, else random quality
  double spacing;

  // switch to random fragments -- uses only minq and maxq from above list
  boolean useRandom = false;
  double minEdge;
  double maxEdge;
  double coverage;  // fraction of world area to be covered in fragments, can be > 1

  /** Regular case:
   *  The subsites are named name+Number, and get site qualities descending
   *  from max to min. The worldSite takes the area of the world. nx and ny give
   *  the number of array
   *  areas in x and y. If the request's arithmetic doesn't work out (spacing
   *  can't be honored) there's simply nx by ny subsites. If random, the
   *  array parameters are unused.
   */
  public FragSite(String name, LimitResourceQuad worldSite,
    int nx, int ny,
    double maxquality, double minquality, double spacing, Color c)
  {
    super(name, worldSite, c);
    maxq = maxquality;
    minq = minquality;
    numx = nx;
    numy = ny;
    numFrags=nx*ny;
    this.spacing = spacing;
  }

  protected void arrayedBirthday() {
    LimitResourceQuad frag;
    Mcr world = Globals.getWorldMcr();
    double worldx = world.getLenX();
    double worldy = world.getLenY();
    if ((spacing * (numx-1)) >= worldx)
      spacing = 0;
    if ((spacing * (numy-1)) >= worldy)
      spacing = 0;

    double sizex = (worldx - ((numx - 1)*spacing)) / numx;
    double sizey = (worldy - ((numy - 1)*spacing)) / numy;
    double nextx = world.lx;
    double nexty = world.ly;
    double deltaq = - (maxq - minq) / (numx*numy - 1);
    double nextq = maxq;
    for (int i=0; i<numx; i++) {
      for (int j=0; j<numy; j++) {
        frag = new LimitResourceQuad();
        frag.setSite(this);
        frag.lx = nextx;
        frag.ly = nexty;
        frag.ux = nextx + sizex;
        frag.uy = nexty + sizey;
        if (gradient) {
          frag.setQuality(maxq, minq, Globals.getWorldMcr());
        } else {
          frag.setQuality(nextq);
        }
        Globals.addLandQuad(frag);
        nextq += deltaq;
        nexty += sizey + spacing;
      }
      nextx += sizex + spacing;
      nexty = world.ly;
    }
  }

  protected void randomBirthday() {
    // make the random collection of boxes
    Vector boxList = RandomMcrs.makeRandomMcrs(Globals.getWorldMcr(),
        coverage, minEdge, maxEdge);
    numFrags = boxList.size();

    // need the same random source that made the boxes, or the
    // quality assignments won't repeat between runs
    Random rsource = RandomMcrs.getLastRandomSource();

    // loop through all the boxes, make corresponding sites in world
    for (int i=0; i<boxList.size(); i++) {
      LimitResourceQuad frag = new LimitResourceQuad();
      frag.setSite(this);
      frag.setFrom( (Mcr) boxList.elementAt(i) );
      if (gradient) {
        frag.setQuality(maxq, minq, Globals.getWorldMcr());
      } else {
        frag.setQuality( minq + ( (maxq - minq) * rsource.nextDouble() ) );
      }
      Globals.addLandQuad(frag);
    }
    boxList.removeAllElements(); // encourage garbage collection
    Globals.log("Used " + numFrags + " fragments.");
  }

  protected void birthday() {
    super.birthday(); // instantiate supersite and do the stat thing
    if (useRandom) {
      randomBirthday();
    } else {
      arrayedBirthday();
    }
  }

  public int getNumX(){return numx;}
  public void setNumX(int v) {numx = v; numFrags=numx*numy;}
  public int getNumY(){return numy;}
  public void setNumY(int v) {numy = v; numFrags = numx*numy;}
  public int getNumFrags() {return numFrags;}
  public double getMaxQuality(){return maxq;}
  public void setMaxQuality(double v) {maxq = v;}
  public double getMinQuality(){return minq;}
  public void setMinQuality(double v) {minq = v;}
  public double getSpacing(){return spacing;}
  public void setSpacing(double v) {spacing = v;}

  /** Gradient overrules arrayed min->max. */
  public void setUseWorldGradient(boolean b) {gradient = b;}

  public double getMinEdge(){return minEdge;}
  public void setMinEdge(double v) {minEdge = v;}
  public double getMaxEdge(){return  maxEdge;}
  public void setMaxEdge(double v) { maxEdge = v;}
  public double getCoverage(){return coverage;}
  public void setCoverage(double v) {coverage = v;}
  public boolean isRandom(){return useRandom;}
  public void setRandom(boolean v) {useRandom = v;}
}
