//Title:        Gecko Woods
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  "mini-Sortie"
/**
 * 991130, vfb, original Java
 * ReaperClearcut is people damage. It is rectangular, and kills every
 * tree it crosses. There's a max of 1.0 clearcut per year. The random
 * edgelength is applied twice, once in x and once in y. The clearcut is
 * placed randomly on the world.
 */


package gecko.woods;
import gecko.*;
import java.awt.*;

public class ReaperClearcut implements Reapable, Steppable {
  private double frequency; // random, % years that decide to do a clearcut
  private double maxEdge;   // max edge length in x or y
  private double minEdge;

  // this year
  private Mcr cutter = new Mcr();
  private boolean clearcutYear; // decided to do a clearcut this year yes/no

  public ReaperClearcut() {
    frequency = maxEdge = minEdge = 0;
    clearcutYear = false;
    cutter.setNullMcr();
  }

  public void step() {
    clearcutYear = Globals.getRandomDouble() < frequency;
    if (Globals.getCurrentTime() == 0)
      cutter.setNullMcr();
    if (clearcutYear) {
      double edgex, edgey;
      Mcr world = Globals.getWorldMcr();
      edgex = (maxEdge - minEdge) * Globals.getRandomDouble();
      edgey = (maxEdge - minEdge) * Globals.getRandomDouble();
      cutter.lx = world.lx + ((world.ux - world.lx - edgex) * Globals.getRandomDouble());
      cutter.ux = cutter.lx + edgex;
      cutter.ly = world.ly + ((world.uy - world.ly - edgey) * Globals.getRandomDouble());
      cutter.uy = cutter.ly + edgey;
      Globals.log("Clearcut, time " + Globals.getCurrentTime() + " area=" + cutter);
    }
  }
  public void reap(ReapableTree tree) {
    if (!clearcutYear) return;
    if (cutter.crossMcr((Mcr)tree))
      tree.die();
  }
  public double getFrequency(){return frequency;}
  public void setFrequency(double v) {frequency = v;}
  public double getMaxEdge(){return maxEdge;}
  public void setMaxEdge(double v) {maxEdge = v;}
  public double getMinEdge(){return minEdge;}
  public void setMinEdge(double v) {minEdge = v;}
}
