//Title:        Gecko Woods
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  "mini-Sortie"
/**
 * 991130, vfb, original Java
 * ReaperHurricane only damages Trees on reap(Tree). It's parameters:
 * Frequency: random chance of any hurricane this year.
 * SeverityMax: if there is a hurricane, a random * severityMax sets
 *   the maximum damage rate (damageMax) for this year's hurricane. When
 *   it comes time to reap, damageMax * random sets how much damage is done
 *   to this particular tree. Damage is taken out of crown area. So, if the
 *   crown's current area is C, the crown radius is reset to match damage*C.
 *   The Tree may be inclined to die as a result, but hurricane doesn't kill them.
 *
 *  Hurricanes are a global effect (the storm is bigger than the world.)
 *  This may be pretty hokey without per-species wind resistance.
 */


package gecko.woods;
import gecko.*;

public class ReaperHurricane implements Reapable, Steppable {
  private double frequency; // random, % years that decide to do a hurricane
  private double severityMax; // random 0..severityMax sets damageMax for this hurricane

  // this year
  private double damageMax; // this year, inflict random 0..damageMax damage on all Trees
  private boolean hurricaneYear; // decided to do a hurricane this year yes/no

  public ReaperHurricane() {
    frequency = 0;
    severityMax = 0;
    hurricaneYear = false;
    damageMax = 0;
  }

  public double getFrequency(){return frequency;}
  public void setFrequency(double v) {frequency = v;}
  public double getSeverityMax(){return severityMax;}
  public void setSeverityMax(double v) {severityMax = v;}

  public void step() {
    hurricaneYear = Globals.getRandomDouble() < frequency;
    if (hurricaneYear) {
      damageMax = Globals.getRandomDouble() * severityMax;
      Globals.log("Hurricane, time " + Globals.getCurrentTime() + " severity " + damageMax);
    }
  }
  public void reap(ReapableTree tree) {
    if (!hurricaneYear) return;
    double damage = damageMax * Globals.getRandomDouble();
    tree.crownDamage(damage);
  }
}
