//Title:        Gecko Woods
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  "mini-Sortie"
/**
 * 991130, vfb, original Java
 * ReaperRoundFire is a circle of fire, kinda like a lightning strike during
 * a rainstorm. The simplest kind of fire. Fires do damage to Trees, but their
 * damage can exceed 1.0 (which kills 'em.) The damage from a fire decreases
 * toward its edge as 1.2 - (dist/maxdist)^2. Parameters:
 *  frequency--there's max one round fire per year per roundfire reaper (but
 *    you can have more than one roundfire reaper)
 *  maxsize--a diameter. The minsize is 0, and thus not specified.
 */


package gecko.woods;
import gecko.*;
import gecko.zoo.QuadCircleSpecies;
import java.awt.*;
import courseware.jfcdep.TruncVal;

public class ReaperRoundFire implements Reapable, Steppable {
  private double frequency; // random, % years that decide to do a clearcut
  private double maxSize;   // max fire diameter

  // this year
  private double x, y, r;
  private boolean fireYear; // decided to do a fire this year yes/no

  public ReaperRoundFire() {
    frequency = maxSize = 0;
    fireYear = false;
    x=y=r=0;
  }

  public void step() {
    fireYear = Globals.getRandomDouble() < frequency;
    if (fireYear) {
      r = (maxSize / 2.0) * Globals.getRandomDouble();
      Mcr world = Globals.getWorldMcr();
      x = world.getLenX() * Globals.getRandomDouble() + world.lx;
      y = world.getLenY() * Globals.getRandomDouble() + world.ly;
      Globals.log("RoundFire, time=" + Globals.getCurrentTime() +
        ", r=" + TruncVal.truncVal(r,4) +
        ", x=" + TruncVal.truncVal(x,4) +
        ", y=" + TruncVal.truncVal(y,4));
    }
  }
  public void reap(ReapableTree tree) {
    if (!fireYear) return;
    QuadCircleSpecies aTree = (QuadCircleSpecies) tree;
    double dist = Globals.distance(x, y, aTree.x, aTree.y);
    if (dist > r)
      return;
    dist /= r;
    double damage = 1.2 - dist*dist;
    if (damage >= 1)
      tree.die();
    else
      tree.crownDamage(damage);
  }
  public double getFrequency(){return frequency;}
  public void setFrequency(double v) {frequency = v;}
  public double getMaxSize(){return maxSize;}
  public void setMaxSize(double v) {maxSize = v;}
}
