//Title:        Gecko Woods
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  "mini-Sortie"
/**
 * 991018, vfb, original Java
 * Some place to put globals for the Woods family of simulators.
 */

package gecko.woods;

public class Woods {
  // Average angle of the sun, in radians, for growing season, at this lattitude
  // For 42N, using 26 degrees, converted to radians.
  static public double sunAngle = Math.PI * 26.0 / 180.0;
  static private double sunSine = Math.sin(sunAngle);
  // we only look for light coming from within maxSunAngle of vertical
  static public double maxSunAngle = Math.PI * 45.0 / 180.0;
  // setting weightDirectSun to other than 1.0, slows down runs
  // by adding in the skylight calculation, which is not well tested -- 000114, vfb.
  // vfb reeval 000626, pretty important in boreal case where almost no direct sun there.
  // not a lot slower: benched both=10min, only sky=9, only Nshadows=8 min, similar pops.
  static public double weightDirectSun = 0; // (1-weight)sky + weight*direct sun
  // this factor x trunk radius determines self-thinning ability of forest
  // for trunk aversion--bigger trees kill saplings within stemR * thinningFactor
  static public double thinningFactor = 10.0;
  static public boolean wrapX = false;
  static public boolean wrapY = false;
  // it matters a great deal to our speed that we use as small a maxHeight as
  // possible....
  static public double maxHeight = 0;
  static public boolean showShadows = true; // draw oval to indicate tree shadow/height
  static public boolean showHeights = true; // draw "h=x" to indicate tree height
  static public boolean diffusiveLightBox = true;
  static public boolean crownCrop = true;
  static public boolean speciesRandomDeath = true;

  static public double getSunAngle(){return sunAngle;}
  static public double getSunSine(){return sunSine;}
  static public void setThinningFactor(double tf){thinningFactor = tf;}
  static public double getThinningFactor(){return thinningFactor;}
  static public void setSunAngle(double radians) {
    sunAngle = radians;
    sunSine = Math.sin(sunAngle);
  }
  static public void setSunAngleD(double degrees) {setSunAngle(degrees * Math.PI / 180.0);}
  static public double getSunAngleD() {return sunAngle * 180.0 / Math.PI; }
  static public void setWeightDirectSun(double weight) {weightDirectSun=weight;}
  static public double getWeightDirectSun() {return weightDirectSun;}
  static public boolean isWrapX(){return wrapX;}
  static public void setWrapX(boolean v) {wrapX = v;}
  static public boolean isWrapY(){return wrapY;}
  static public void setWrapY(boolean v) {wrapY = v;}
  // remember to reset to zero each run for speed! Tree will grow it.
  static public double getMaxHeight(){return maxHeight;}
  static public void setMaxHeight(double radians) {maxHeight = radians;}
  static public boolean isShowShadows(){return showShadows;}
  static public void setShowShadows(boolean v) {showShadows = v;}
  static public boolean isShowHeights(){return showHeights;}
  static public void setShowHeights(boolean v) {showHeights = v;}
  static public boolean isDiffusiveLightBox(){return diffusiveLightBox;}
  static public void setDiffusiveLightBox(boolean v) {diffusiveLightBox = v;}
  static public boolean isCrownCrop(){return crownCrop;}
  static public void setCrownCrop(boolean v) {crownCrop = v;}
  static public boolean isSpeciesRandomDeath(){return speciesRandomDeath;}
  static public void setSpeciesRandomDeath(boolean v) {speciesRandomDeath= v;}

  /** Will later wish to wobble GLI. */
  static public double getGLI() {
    return 100.0;
  }

}
