//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * 990625, vfb, original Java
 * Translated from the Swarm Objective-C Gecko source, version 0.51
 * With structural changes.
 */
/** Carnie (carnivore/spider) from Gecko--min/kid/breed radii.
 */

package gecko.zoo;

import gecko.*;
import java.awt.Color;
import courseware.jfcdep.*;

public class Carnie extends SizedPlant {
  private double smellRadius;		// multiplier on radius for smelling distance
  private int sleepTime;		// how long it takes to assimilate a grazer
  private int sleepLeft;		// rounds left before wake up.
  private long lastAte;	  	// getCurrentTime() at last meal
  private double hibernateTax;		// taxrate while hibernating
  private int hibernateRule;	// hibernate if haven't eaten in this long
  private int awakeColor = QuadDisplay.setColor("Carnie Hunting", Color.red);
  private int sleepColor = QuadDisplay.setColor("Carnie Sleeping", Color.magenta);

  private double defConversion = .56;
  private double defTax = .127;
  private double defTaxExp = .75;
  private double defBreedSize = 7.18;
  private double defKidSize = 4.04;
  private double defMinSize = 2.52;
  private double defSmellRadius = 2.09;
  private double defHibernateTax = .0072;
  private int defSleepTime = 6;
  private int defHibernateRule = 15;

  // avoid allocating the silly little things
  private static McrCircle huntwin = new McrCircle();

  // just makes picks easier to read
  private final static boolean doAttack = true;
  private final static boolean noAttack = false;
  /** This property list is used to generate paramLists for popup editing.
   *  Note initArea is left off for now, as don't have the routines to set
   *  it nicely from a popup.
   */
  private static String[] myprops = { "smellRadius", "sleepTime",
     "hibernateRule", "hibernateTax" };
  protected static String[] props;
  static {
    String[] firsthalf = SizedPlant.props;
    props = new String[firsthalf.length + myprops.length];
    for (int i=0; i<myprops.length; i++) {
      props[i] = myprops[i];
    }
    for (int i=0; i<firsthalf.length; i++) {
      props[myprops.length+i] = firsthalf[i];
    }
  }
  public String[] getPropertyNames() { return props;}


  public Carnie() {
    super();
    init();
  }

  public void init() {
    setPutOnPlantRoot(false); // go on mobileRoot
    double angle = 2 * Math.PI * Globals.getRandomSource().nextDouble();
    direction.setDirection(angle);
    super.init();
    sleepTime = 0;
    sleepLeft = 0;
    lastAte = Globals.getCurrentTime();
    getMetabolism().setIeat(GrassResources.RESOURCE_PROTEINS);
    getMetabolism().setIam(GrassResources.RESOURCE_CPROTEINS);
    getMetabolism().setConversion(GrassResources.RESOURCE_PROTEINS,
                         GrassResources.RESOURCE_CPROTEINS, defConversion);
    getMetabolism().setTax(GrassResources.RESOURCE_CPROTEINS, defTax, defTaxExp);
    setBreedSize(defBreedSize);
    setKidSize(defKidSize);
    setMinSize(defMinSize);
    setSmellRadius(defSmellRadius);
    setScaleRadius(1.0);
    setScaleVolume(1.0);
    setHibernateTax(defHibernateTax);
    setSleepTime(defSleepTime);
    setHibernateRule(defHibernateRule);
  }

  public double getSmellRadius(){return smellRadius;}
  public void setSmellRadius(double sr){smellRadius=sr;}
  public int getSleepTime(){return sleepTime;}
  public void setSleepTime(int i){sleepTime=i;}
  public int getSleepLeft(){return sleepLeft;}
  public void setSleepLeft(int i){sleepLeft=i;}
  public long getLastAte(){return lastAte;}
  public void setLastAte(long i){lastAte=i;}
  public double getHibernateTax(){return hibernateTax;}
  public void setHibernateTax(double d){hibernateTax=d;}
  public int getHibernateRule(){return hibernateRule;}
  public void setHibernateRule(int i){hibernateRule=i;}
  public void setSleepColor(String name, int r, int g, int b) {
    sleepColor = QuadDisplay.setColor( name, new Color(r, g, b) );
  }
  public void setColor(String name, int r, int g, int b) {
    super.setColor(name, r, g, b);
    awakeColor = getColor();
  }

  public double getMoveRadius() { return getRadius();}
  public double getPerceiveRadius() { return getRadius() * smellRadius;}

  public void setFrom(Carnie other) {
    super.setFrom(other);
    setSmellRadius(other.getSmellRadius());
    setSleepTime(other.getSleepTime());
    setHibernateTax(other.getHibernateTax());
    setHibernateRule(other.getHibernateRule());
    setSleepLeft(0);
    setLastAte(Globals.getCurrentTime());
    double dirAngle = 2 * Math.PI * Globals.getRandomSource().nextDouble();
    direction.setDirection(dirAngle);
    sleepColor = other.sleepColor;
    awakeColor = other.awakeColor;
  }

  public Object clone() {
    return makeKid();
  }

  public Speciesable makeKid() {
    Carnie kid = new Carnie();
    kid.setFrom(this);
    getSpecies().notifyBirthToMom(this);
    return kid;
  }

  /** Look for prey--arbitrary logic, so localize. Core of Carnie behavior. */
  protected Critter findCritter(double distance, boolean attack, Quad huntroot) {
    final boolean noPickInvis = false;
    Quad other, bestother;
    Critter odata, bdata;

    // hide myself from picks
    setVisible(false);

    // find first candidate
    bestother = null;
    huntwin.circleToSquare(getX(), getY(), distance);
    other = huntroot.pickWin(huntwin, huntroot, noPickInvis);

    // if found anything...
    if (other != null)
      // loop through what found....
      do {
        odata = (Critter) other;
        // how to look if looking to attack
        if (attack) {
          if (this.attacks(odata)) {
            if (bestother!=null) {
              bdata = (Critter) bestother;
              // looking for fattest prey
              if (bdata.getRadius() > odata.getRadius())
                bestother = other;
            } else {
              bestother = other;
            }
          }
        // how to look if looking for whatever's around
        } else {
          if (!this.attacks(odata)) {
            if (bestother!=null) {
              // rather than fattest, just 50/50 chance this is the "found"
              // note that means this is NOT the way to consider all neighbors!
              // we're looking here for "Best", whatever that is
              if (Globals.getRandomSource().nextDouble()  > 0.5)
                bestother = other;
            } else {
              bestother = other;
            }
          }
        }
        
        other = other.pickWin(huntwin, huntroot, noPickInvis);
    } while (other!=null);

    // done picking, stop hiding from picks
    setVisible(true);
    return (Critter) bestother;
  }


  public void move() {
    double[] direx;
    double delta = getMoveRadius();

    if (sleepLeft > 0)
      return;
    if (delta == 0.0)
      return;

    direx = direction.getDirection();
    direx[0] *= Math.abs(delta);
    direx[1] *= Math.abs(delta);

    setX(getX() + direx[0]);
    setY(getY() + direx[1]);
    direction.setDirection(direx[0], direx[1]);
    setMcr();
  }

  /** removed from hunt for subclass convenience. */
  protected void doEat(Critter gopper) {
    double dinner;
    int meat = getMetabolism().getIeat();
    
    dinner = gopper.stealAllResource(meat);
    feed(meat, dinner);
    direction.setDirection(gopper.getX() - this.getX(), gopper.getY() - this.getY());
    // the gopper steps? verbatim from Gecko 0.51...I think this means "die"...
    gopper.step();
    setSleepLeft(sleepTime);
    setLastAte(Globals.getCurrentTime());
    setColor(sleepColor);
  }

  /** Pretty complex behavior for carnies.  Priorities:
   * 	1. try to eat (any gopper will do)
   * 	2. move away from a fellow Carnie.
   * 	3. move toward something to eat (as is, any gopper)
   * 	4. if not on a plant, move toward any plant
   * Transcription note: in the original, this had a flag indicating whether
   * a rational move was found, so that if not, a random move could be taken.
   * But the flag was never consulted--never did random move. So, removed the
   * flag. If no rational move found, I guess it doesn't move.
   */
  public void hunt() {
    Critter plant;
    Critter gopper;
    Critter carnie;

    // Can eat a gopper?
    gopper = findCritter(getRadius(), doAttack, Globals.getMobileRoot());
    if (gopper != null) {
      doEat(gopper);

    // no gopper to eat--move away from other carnie? cuts down on competition
    } else {
      carnie = findCritter(getPerceiveRadius(), noAttack, Globals.getMobileRoot());
      // this was getSpecies rather than getClass in Gecko 0.51
      if ( (carnie!= null) && (carnie.getClass() == this.getClass()) ) {
        direction.setDirection(this.getX() - carnie.getX(), this.getY() - carnie.getY());

      // Next choice is to head toward another gopper I can smell but can't reach yet.
      } else {
          gopper = findCritter(getPerceiveRadius(), doAttack, Globals.getMobileRoot());
          if (gopper != null) {
            direction.setDirection(gopper.getX() - this.getX(), gopper.getY() - this.getY());

       // well, what else is in the neighborhood?
       // if not on a plant, better move toward one, since that's where the goppers are
       } else {
          plant = findCritter(getRadius(), noAttack, Globals.getPlantRoot());
          if (plant == null) {
            plant = findCritter(getPerceiveRadius(), noAttack, Globals.getPlantRoot());
            if (plant != null) {
               direction.setDirection(plant.getX() - this.getX(), plant.getY() - this.getY());
            }
          }
        }
      }
    }
  }

  /** Carnies go into hibernation if they haven't eaten lately.*/
  public boolean isHibernating() {
    if ( Globals.getCurrentTime() - getLastAte() > getHibernateRule() ) {
      setColor(sleepColor);
      return true;
    } else return false;
  }

  /** For paying abnormal taxes. */
  public void paytaxRate(double rate) {
    int mybody = getMetabolism().getIam();
    double saveTax = getMetabolism().getTaxRate(mybody);
    getMetabolism().setTax(mybody, rate, getMetabolism().getTaxExponent(mybody));
    super.paytax();
    getMetabolism().setTax(mybody, saveTax, getMetabolism().getTaxExponent(mybody));
  }

  public boolean isSleeping() {
    return (sleepLeft > 0) || isHibernating();
  }

  /** Override Critter-step because of sleep/wakeup and plantcycle.
   *  Transcription note: there was a carnieCycle in Gecko 0.51....skip since
   *  didn't appear used in any code or protocol.
   */
  public void step() {
    Critter other;

    if ( (Globals.getCurrentTime() > 0) && (getAge() == 0))
      return;

    // sleep step
    if ( isSleeping() ) {

      // first check if gopper stepped on me and woke me up.
      // if does, have to pay all metabolic taxes due for whole sleep, i.e.,
      // don't get as much out of last meal as we should have had we digested well.
      other = findCritter(getPerceiveRadius(), doAttack, Globals.getMobileRoot());
      if (other != null) {
        while (sleepLeft > 0) {
          if (!paytax()) {
            return;
          }
          sleepLeft--;
        }
        // Though isn't true, it makes me stop hibernating.
        lastAte = Globals.getCurrentTime();
      } else {
        if (isHibernating()) {
          paytaxRate(hibernateTax);
          return;
        } else {
          paytax();
          sleepLeft--;
          return;
        }
      }
    }

    // awake step
    setColor(awakeColor);

    super.step();

    // this routine both checks and sets hibernating mode
    if (isHibernating()) {};
  }

  public String toString() {
    return "Carnie[isHibernating=" + isHibernating() +

            ",smellRadius="+smellRadius+",sleepTime="+sleepTime+

            ",hibernateRule="+hibernateRule+

            ",hibernateTax="+hibernateTax+

           ",super=" + super.toString() + "]";

  }

}