//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator
/**
 * 990809, vfb, original Java
 * Translated from the Swarm Objective-C Gecko source, version 0.51
 * With structural changes.
 */
/** Notes from the original:
  * original, 970123, vfb, for Os experiment comparing effect of predation
  * (death) vs. predation risk (evasive behavior) on ecosystem dynamics.
  * note:  it's possible to have a scaredyGrazer that's not afraid of anything.
  * just set scaredof to nil (or blank on protocol)

  * seasonal modifications, 970701, vfb, for same Os experiment:
  * Iff there are seasons (geckoModelSwarm->settings > 0), then newborns are
  * laid dormant. For esthetics, they're visible, but all deleted at the
  * end of the season, and their number used as InitNum next season.
 */


package gecko.zoo;
import gecko.*;
import java.util.Random;
import courseware.jfcdep.TruncVal;

public class ScaredyGrazer extends TimeLimitGrazer implements Exposureable {
  // defining
  private double smellRadius;		  // multiplier on radius for smelling predators
  private double refugeRadius;	  // multiplier on radius for finding refuge
  private Species refuge;		      // what species plant hides me from predators
  private boolean fearless;		    // if fearless, no evading things that eat me
  private double refugeGrowRate;	// growth rate eating refuge species

  // working
  protected double lastPredatorDistance;
  protected double wannadistance;	// like wannax and wannay, maybe < radius
  protected int mood;		          // our fright/happy/egg status
  // NOTE THIS STATIC! situation is shared, and only valid during my step.
  static protected ScaredySituation situation = new ScaredySituation();

  // mood values
  protected final static int SCAREDY_HAPPY = 0;
  protected final static int SCAREDY_FROZE = 1;
  protected final static int SCAREDY_RUN   = 2;
  protected final static int SCAREDY_EGG   = 3;  // not a mood, but pretty oblivious....

  public int getMood() {return mood;} // for debug

  protected static String[] myprops = { "smellRadius", "refugeRadius",
    "refugeGrowRate", "fearless" };
  protected static String[] props;
  static {
    String[] firsthalf = TimeLimitGrazer.props;
    props = new String[firsthalf.length + myprops.length];
    for (int i=0; i<myprops.length; i++) {
      props[i] = myprops[i];
    }
    for (int i=0; i<firsthalf.length; i++) {
      props[myprops.length+i] = firsthalf[i];
    }
  }
  public String[] getPropertyNames() { return props;}


  public ScaredyGrazer() {
    super();
    refuge=null;
    fearless = false;
    mood = SCAREDY_HAPPY;
  }

  public void setFrom(ScaredyGrazer other) {
    super.setFrom(other);
    smellRadius    = other.smellRadius;
    refugeRadius   = other.refugeRadius;
    refugeGrowRate = other.refugeGrowRate;
    refuge         = other.refuge;
    fearless       = other.fearless;
  }

  public Speciesable makeKid() {
    ScaredyGrazer kid = new ScaredyGrazer();
    kid.setFrom(this);
    kid.initKidMood();
    getSpecies().notifyBirthToMom(this);
    return kid;
  }

  public void initKidMood() {
    if (Globals.isSeasonal()) { // if there -are- seasons only
      // if this isn't the species birthday, but rather an interim kid
      if ( !getSpecies().isBirthday() )
        mood = SCAREDY_EGG; // born to sit and do nothing.
      else
        mood = SCAREDY_HAPPY;
    } else
      mood = SCAREDY_HAPPY;
  }
  
  public Object clone() {
    return makeKid();
  }

  /** This is a multiplier on current radius, not an absolute. */
  public double getSmellRadius() { return smellRadius;}
  public void setSmellRadius(double r) {smellRadius = r;}
  public double getRefugeRadius() { return refugeRadius;}
  public void setRefugeRadius(double r) {refugeRadius = r;}
  public double getRefugeGrowRate() { return refugeGrowRate;}
  public void setRefugeGrowRate(double r) {refugeGrowRate = r;}
  /** When VaryProperty is told to vary radius, it needs to set minSize
   *  and kidSize down if they're higher than the new radius. This could
   *  have side effects... Decided to only check on birthday, and set kid
   *  and min as fixed offsets from radius. E.g., if kidSize was offset 0
   *  from radius before, it's offset 0 from radius after, and likewise for
   *  minSize.
   */
  public void setRadius(double d) {
    // save off current offset
    double offsetMin = minsize - radius;
    double offsetKid = kidsize - radius;
    // let super do its thing
    super.setRadius(d);
    if (getSpecies().isBirthday()) {
      setMinSize(d + offsetMin);
      setKidSize(d + offsetKid);
    }
  }


  public void setRefuge(Species s) {refuge = s;}
  public Species getRefuge() { return refuge;}
  public void setFearless(boolean b) {fearless = b;}
  public boolean isFearless() { return fearless;}

  public String toString() {
    String ref = (refuge == null) ? "none" : refuge.getName();
    return "ScaredyGrazer(fearless=" + fearless +
    ",refuge=" + ref +
    ",smellR=" + smellRadius +
    ",refugeR=" + refugeRadius +
    ",refugeGR=" + refugeGrowRate +
    ",super=" + super.toString();
  }


  /** am I entirely hidden by refuge plants? If exposed (part or whole unhidden)
   *  return the Mcr of the exposed area. If entirely hidden, isNullMcr.
   *
   *  exposure's return Mcr (aside from isNullity) is actually for predator use
   *
   *  these routines significantly rewritten from the Objective-C, which I
   *  never was very happy with.
   *
   *  vfb, 000128--this doesn't go into ScaredySituation because it's called
   *  from Carnie, thus not during the window that ScaredySituation is set to
   *  match this grazer. We could remember this as part of our state, but I'd
   *  rather allow hidden to become untrue between my step and the Carnie's
   *  (because 5 of us are hiding in the same plant, etc.)
   */
  public Mcr exposure() {
    McrCircle exposed = new McrCircle(); // new Mcr isNullMcr.
    // eggs are never "exposed". Buried, perhaps. :)
    if (mood == SCAREDY_EGG)
      return exposed;
    Quad pq;
    Critter plant;
    int hidingpower;   // an mcr-relation
    Quad plantRoot = Globals.getPlantRoot();

    feedwin.circleToSquare(this);

    pq = plantRoot.pickWin(feedwin, plantRoot);
    if (pq != null) {
      do {
        plant = (Critter) pq;
        if (plant.getSpecies() == refuge) {
          hidingpower = feedwin.unIntersectMcr(pq, exposed);
          // easiest case, I'm entirely contained within this plant
          if (hidingpower == MCR_CONTAINEDBY) {
            exposed.setNullMcr();
            return exposed;
          } else if (hidingpower != MCR_DISJOINT) {
            // error check for roundoff-sized sliver exposure
            double xerr = exposed.ux - exposed.lx;
            double yerr = exposed.uy - exposed.ly;
            double threshold = radius/1000.0;
            if ((xerr>threshold) && (yerr>threshold)) // neither should be negative, so no abs here
            feedwin.setMcr(exposed);
            else {
              exposed.setNullMcr();
              return exposed;
            }
          }
          // now, either the plant didn't help me or the
          // feedwin's been reset to reflect partial cover
        }
        pq = pq.pickWin(feedwin, plantRoot);
      } while (pq != null);
    }
    exposed.setMcr(feedwin);

    return exposed;
  }

  public boolean amHidden() {

    return exposure().isNullMcr();
  }


  protected final int RUN_FROM_THREAT = 0;
  protected final int RUN_TO_SAFETY = 1;
  protected final int RUN_TO_BEST = 2;
  protected final int RUN_TO_FAR_BEST = 3;

  /** this routine assumes that what you're running to/from exists...
   *  so figure your options before calling this.
   */
  protected void setIntendedMove(int runcode) {
    switch (runcode) {
      case RUN_FROM_THREAT :
        setDirection(x - situation.threat.x, y - situation.threat.y);
        break;
      case RUN_TO_SAFETY :
        setDirection(situation.safePlant.x - x, situation.safePlant.y - y);
        // because safeD is edge-to-edge and we want to just slip inside.
        // note that if we're already inside (or partly inside) this is
        // flipping us out to the edge.... so don't runtosafety if safe!
        wannadistance = situation.safeD + 2.0*radius;
        break;
      case RUN_TO_BEST :
        setDirection(situation.yumPlant.x - x, situation.yumPlant.y - y);
        break;
      case RUN_TO_FAR_BEST :
        setDirection(situation.farYumPlant.x - x, situation.farYumPlant.y - y);
        break;
    }

    limitMove();
  }

  // apply limits--move this out of getDesiredMove so that I never overlook
  // it again....  But localize for OferStateGrazer to override.
  private void limitMove() {
    if (wannadistance <= 0.0)
      wannadistance = 0;
    else if (wannadistance > (2.0 * getRadius()) )
	    wannadistance = 2.0 * getRadius();
  }

  /** set to mill around (finding no food may override this choice) */
  protected void setIntendedMoveRandom() {
    double angle, veerangle;
    lastPredatorDistance = Double.POSITIVE_INFINITY;
    angle = dirAngle;
    veerangle = (getVeer()/2.0) * Globals.getRandomSource().nextDouble();
    if (Globals.getRandomSource().nextDouble() < 0.5)
      veerangle = - veerangle;
    angle += veerangle;
    setDirection(angle);
  }

  protected void runOrHide()
  {
    lastPredatorDistance = situation.threatD;
    if (situation.safePlant != null)
      setIntendedMove(RUN_TO_SAFETY);
    else
      setIntendedMove(RUN_FROM_THREAT);
  }

  /** find what RADIUS of plant will satisfy my growth and tax desires.
   *  Depends on the growRate associated with preferred and refuge plants,
   *  so returns different bite sizes.
   *  vfb, 000128, Public because ScaredySituation needs it.
   */
   public double calcBite(double growRate) {
    int meResource = getMetabolism().getIam();
    double mybite;
    double biteR;
    double converts;
    double myvol; // my body volume
    Metabolism m = getMetabolism();

    myvol   = m.getStores(meResource);
    mybite  = m.getTaxRate(meResource)
              * Math.pow( myvol, m.getTaxExponent(meResource) );
    mybite += growRate * myvol;
    converts = m.getConversion(m.getIeat(), meResource);
    mybite /= converts;

    biteR = Globals.volume2radius(mybite, 1.0, getScaleVolume());
    return biteR;
  }

  /** return % ideal meal I managed to get. */
  protected double eat() {
    Critter plant;
    double gotpercent = 0; // percent of my ideal I managed to eat

    if (situation.yumPlant != null) {
      // eat where we are
      gotpercent = eatIt();
    } else if (situation.farYumPlant != null) {
      setIntendedMove(RUN_TO_FAR_BEST);
    }
    double idealVol = Globals.radius2volume(situation.normalBiteR, 1.0, getScaleVolume());
    double breakeven = calcBite(0); // no growth R
    breakeven = Globals.radius2volume(breakeven, 1.0, getScaleVolume());

    gotpercent = (gotpercent - breakeven) / (idealVol - breakeven);
    if (gotpercent < 0) gotpercent = 0;
    return gotpercent;
  }

  /** returns volume eaten */
  private double eatIt() {
    Critter lunch = situation.yumPlant;
    if (lunch == null) return 0;
    double r;
    double vol;
    int meat = getMetabolism().getIeat();

    if (lunch.getSpecies() == refuge)
      r = situation.refugeBiteR;
    else
      r = situation.normalBiteR;

    vol = Globals.radius2volume(r, 1.0, getScaleVolume());

    if (lunch.getRadius() > r) {
      lunch.stealResource(meat,vol);
    } else {
      vol = lunch.stealAllResource(meat);
    }
    lunch.resetRadius();  // since another may choose to eat him.
    this.feed(meat, vol);
    return vol;
  }

  /**
   *  How much I want to eat depends on -my- size & tax debt & growthrate.
   *  Hunt for any plant that satisfies that desire, settle for smaller
   *  one if can't have what we want.
   *  For ScaredyGrazer, much complicated by evasive action.
   *  My plant options and threats are stored in situation. 
   */
  public void hunt() {
    // eggs don't hunt
    if (mood == SCAREDY_EGG)
      return;

    // ---------------------------------------------------------------
    // 1. set mood and intended next move based on current perceptions
    // ---------------------------------------------------------------
    // this looks for all my options and threats
    situation.lookAround(this, fearless);
    wannadistance = this.getRadius(); // unless proven otherwise

    if (situation.threat == null) {
      mood = SCAREDY_HAPPY;
      setIntendedMoveRandom();

    // if I am threatened, my actions (including whether I get to eat)
    // depend on a. my mood (experience last step) and b. refuge availability
    } else {
      // rem eggs exited at first line of routine
      switch (mood) {

        case SCAREDY_HAPPY :
          if (situation.amHidden) {
            mood = SCAREDY_RUN;
            runOrHide();
          } else { // threatened and not safe
            mood = SCAREDY_FROZE;
            lastPredatorDistance = situation.threatD;
            wannadistance = 0.0;
            return;                                       // MISS DINNER
          }
          break;

        case SCAREDY_FROZE :
          mood = SCAREDY_RUN;
          if (situation.amHidden) {
            runOrHide();
          } else {
            // is threat closer than when I froze?
            if (situation.threatD < lastPredatorDistance) {
              lastPredatorDistance = situation.threatD;
              runOrHide();
              return;                                     // MISS DINNER
            } else { // threat not closer, just run
              lastPredatorDistance = situation.threatD;
              // does not run to safe plant, just away from threat.
              setIntendedMove(RUN_FROM_THREAT);
              return;                                     // MISS DINNER
            }
          }
          break;

        case SCAREDY_RUN :
          // just keep doing the best I can
          lastPredatorDistance = situation.threatD;
          runOrHide();
          break;
      }
    }

    // -------------------------------------------------------------------
    // 2. figure out what/how much to eat, if I didn't already miss dinner
    // -------------------------------------------------------------------
    eat();
  }

  /** Need to override because super assumes we'll always move (untrue--we freeze)
   *  by distance radius (untrue--we sometimes snap onto refuge plants.)
   */
  public Move getDesiredMove() {
    double[] unit = direction.getDirection();
    return new Move(wannadistance, unit[0], unit[1]);
  }

  /** Override step only to ensure eggs do -absolutely- nothing. Skip out on
   *  eggs, then call super.step.
   */
  public void step() {
   long today = Globals.getCurrentTime();
   // if one timestep past birthday for season, resetInitNum
   if ( getSpecies().isBirthday() ) {
      getSpecies().setWorkingInitNumber(0);
   }

   if ( (today > 0) && (getAge() == 0) )
     return;

   if (mood == SCAREDY_EGG) {
      if (seasonCheck()) {} ; // just checking it does what must be done.
      getSpecies().notifyBiomass(getMetabolism().getStores(getMetabolism().getIam()));
      getSpecies().notifyAlive(this); // for stages
      return;
    }

    super.step();
  }

  /** If there are seasons, onSeason all scaredy's die. The eggs are counted
   *  and used to set InitNum for the Species to birthday when it comes time
   *  in the next season.
   */
  public boolean onSeason() {
    Species s = getSpecies();
    if (mood == SCAREDY_EGG) {
      if (s.getInitNumber() == 0)
        getSpecies().resetVaryRadius();
      s.setWorkingInitNumber(s.getInitNumber() + 1);
    }
    die();
    return true; // died
  }
}









