//Title:        Gecko
//Version:
//Copyright:    Copyright (c) 1999, All rights reserved
//Author:       Ginger Booth, gingerbooth@cyber-wizard.com, ginger@gbooth.com
//Company:      Yale IBS/CCE
//Description:  Individual-Based Ecology Simulator

/** Generic statistic buckets. Basically, "things" are counted or summed in
 *  buckets. From time to time, a statcycle ends and sums-in-progress are
 *  transfered to last-completed-stats (possibly with processing, like taking
 *  an average), the sums-in-progress are zeroed, and new sums begin to
 *  accummulate. Used in conjunction with StatBucketBroker to collect some
 *  logically related stats and pipe them onto a plot.
 */

package stats;

public class StatBuckets extends StatSource {
  private int numBuckets;
  private double[] inBucket;  // collecting (in-progress, raw) stats
  private double[] outBucket; // completed (derived) stats

  public StatBuckets(int numBuckets) {
    this.numBuckets = numBuckets;
    inBucket = new double[numBuckets];
    outBucket = new double[numBuckets];
  }

  /** Simply copies inBucket to outBucket and clears the inBucket. Any
   *  derivation/processing for outBucket to be complete statistics needs
   *  to happen afterwards.
   */
  public void step() {
    for (int i=0; i<numBuckets; i++) {
      outBucket[i] = inBucket[i];
      inBucket[i] = 0;
    }
  }

  public int getNumBuckets(){return numBuckets;}
  public void addToBucket(int which, double quantity) {inBucket[which] += quantity;}
  public void incrementBucket(int which) {inBucket[which] += 1.0;}
  /** You can get and set only the outbuckets. Setting inBuckets is done via
   *  addToBucket or incrementBucket or overrideInBucket.
   */
  public double getBucket(int which) {return outBucket[which];}
  /** Used to manipulate the derived stat data directly. */
  public double[] getBuckets() {return outBucket;}
  public void setBucket(int which, double amount) {outBucket[which] = amount;}
  public void overrideInBucket(int which, double amount) {inBucket[which] = amount;}
}
